@@ -10,7 +10,7 @@ import pandas as pd
 import numpy as np 
 import mt5se.backtest as backtest
 import random
-from math import *
+#from math import *
 from datetime import datetime
 from datetime import timedelta
 # importamos o módulo pytz para trabalhar com o fuso horário
@@ -36,14 +36,12 @@ H1=3 # 1 hour bars
 
 
 
-"""
+
+def connect(account=None,passw=None,mt5path=None):
+    """
    Connects to the specificied account, password and specifiec (by path) metatrader installation 
         if parameters are not specified it connects to last used account and password
 """
-def connect(account=None,passw=None,mt5path=None):
-	#if not se.connect():
-	#print(“Error on connection”, se.last_error())
-	#exit():
     if account is None and passw is None:
         if mt5path is None:
             res= mt5.initialize()
@@ -63,7 +61,9 @@ def connect(account=None,passw=None,mt5path=None):
             print('Error trying to connect to account: ',account, ' Error code:',mt5.last_error())
         return False
     info=mt5.account_info()
-    if info.margin_so_mode !=mt5.ACCOUNT_MARGIN_MODE_RETAIL_NETTING:
+    if info.margin_so_mode !=mt5.ACCOUNT_MARGIN_MODE_RETAIL_NETTING and info.margin_so_mode !=mt5.ACCOUNT_MARGIN_MODE_EXCHANGE:
+        # ACCOUNT_MARGIN_MODE_RETAIL_NETTING is usually used for simulated accounts
+        # ACCOUNT_MARGIN_MODE_EXCHANGE is usually used for real accounts
         print("It is NOT netting, but the stock exchange should be netting trade mode!! Error!!")  # B3 is also Netting!!
         return False
     #elif info.margin_so_mode ==mt5.ACCOUNT_MARGIN_MODE_RETAIL_HEDGING:
@@ -85,7 +85,9 @@ def connect(account=None,passw=None,mt5path=None):
         platform=ac.name
         connected=True
     return res
-"""
+
+def terminal_info():
+    """
     Returns info regarding the terminal in a dicitionary 
         For instance:
         'community_account': False,
@@ -112,13 +114,14 @@ def connect(account=None,passw=None,mt5path=None):
         'commondata_path': 'C:\\Users\\...\\AppData\\Roaming\\MetaQuotes\\Terminal\\Common
 
 """
-def terminal_info():
     if not connected:
         print("In order to use this function, you must be connected to the . Use function connect()")
         return
     term_info = mt5.terminal_info()
     return term_info
-"""
+
+def account_info(): 
+    """
     Returns info about the account as a dicitionary
     For instance:
   login=25115284
@@ -131,13 +134,13 @@ def terminal_info():
   margin_mode=2
   currency_digits=2
   fifo_close=False
-  balance=99511.4
+  balance=9...
   credit=0.0
   profit=41.82
-  equity=99553.22
-  margin=98.18
-  margin_free=99455.04
-  margin_level=101398.67590140559
+  equity=9.
+  margin=9..
+  margin_free=9..
+  margin_level=1..
   margin_so_call=50.0
   margin_so_so=30.0
   margin_initial=0.0
@@ -151,50 +154,52 @@ def terminal_info():
   company=MetaQuotes Software Corp.
 
 """
-def account_info(): 
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
     account_info = mt5.account_info()
     return account_info
 
-"""
-    returns the current number of assets of the given symbol
-"""
+
 def get_shares(symbolId):
+    """
+    Returns the current number of assets of the given symbol.
+        It will be negative if there is a short position! and zero if there is no position
+"""
     global inbacktest
     global bts
     if inbacktest:
-        print('Esta em backtest. bts=', bts)
+        #print('Esta em backtest. bts=', bts)
         return backtest.get_shares(bts,symbolId)
     #else:
       #  print('NAO esta em backtest')
-
-
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
     pos= mt5.positions_get(symbol=symbolId)
     if pos is not None and pos!=():
         d=pos[0]._asdict() 
-        return d['volume']
+        if d['type']==1:
+            return -1*d['volume']
+        else:
+            return d['volume']
     else:
         return 0
 
-    return pos['volume']
 
 
-"""
+
+def is_market_open(asset='B3SA3'):
+    """
   It returns if the market is open or not for new orders.
      Note that markets can close in different times for different assets, therefore
      you need to inform the target asset. The default target assets is B3 stock (it will work only in B3 stock exchange)
      For other exchanges inform a valid asset ticker (For instance, GOOG should work for Nasdaq).
      It there is no tick for 60 seconds, the market is considered closed!
 """
-def is_market_open(asset='B3SA3'):
-  if not connected:
-    print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
-    return
+    if not connected:
+        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
+        return
    # si=mt5.symbol_info(asset)
    # if siis not None:
   #      if si.trade_mode==mt5.SYMBOL_TRADE_MODE_FULL: # it does not work in XP/B3 (always True)
@@ -202,25 +207,26 @@ def is_market_open(asset='B3SA3'):
   #      else:
   #          return False
   #  return False
-  mt5.symbol_select(asset) # it makes sure that the symbol is present in Market Watch View
-  t_secs=mt5.symbol_info_tick(asset).time # time in seconds
-  now_dt=datetime.now(etctz)+timedelta(hours=-3)
-  last_tick_dt=datetime.fromtimestamp(t_secs,etctz)
-  #print(last_tick_dt)
-  #print(now_dt)
-  if now_dt>last_tick_dt+timedelta(seconds=60):
-      return False
-  else: 
-      return True
+    mt5.symbol_select(asset) # it makes sure that the symbol is present in Market Watch View
+    t_secs=mt5.symbol_info_tick(asset).time # time in seconds
+    now_dt=datetime.now(etctz)+timedelta(hours=-3)
+    last_tick_dt=datetime.fromtimestamp(t_secs,etctz)
+    #print(last_tick_dt)
+    #print(now_dt)
+    if now_dt>last_tick_dt+timedelta(seconds=60):
+        return False
+    else: 
+        return True
 
 
-"""
-It returns the today datetime (hours=minutes=seconds=0)
+
+def today(offset=None):
+    """
+Returns the today datetime (hours=minutes=seconds=0)
  You can define an offset in days.
     today(1) returns tomorrow
     today(-1) returns yesterday
 """
-def today(offset=None):
     dt=datetime.now()
     dt=datetime(dt.year,dt.month,dt.day,0,0,0)
     if offset is None:
@@ -232,17 +238,18 @@ def today(offset=None):
         dt=dt+timedelta(days=offset)
         return dt
 
-"""
-It returns the current datetime 
+
+def now(dayOffest=None,hourOffset=None,minOffset=None):
+    """
+    Returns the current datetime 
  You can define an offset in days,hours and minutes
     today(dayOffset=1) returns tomorrow
     today(dayOffset=-1) returns yesterday
     today(dayOffset=-1,hourOffset=-2,minOffset=-30) returns yesterday two hours and 30 minutes 
 """
-def now(dayOffest=None,hourOffset=None,minOffset=None):
     dt=datetime.now()
     if dayOffest is not None:
-        dt=dt+timedelta(days=offset)
+        dt=dt+timedelta(days=dayOffest)
     if hourOffset is not None:
         dt=dt+timedelta(hours=hourOffset)
     if minOffset is not None:
@@ -264,35 +271,37 @@ def now(dayOffest=None,hourOffset=None,minOffset=None):
   #  return False
 
 
-"""
+
+def get_volume_step(assetId):
+    """
   Returns the volume step for an asset
 """
-def get_volume_step(assetId):
     step=mt5.symbol_info(assetId).volume_step
     return step
     
-"""
-    returns the max volume of shares thay you can buy, with your balance
+
+def get_affor_shares(assetId,price,money=None,volumeStep=None):
+    """
+    Returns the max volume of shares thay you can buy, with your balance
         it also observes the volume step (a.k.a minimum number of shares you can trade)
 """
-def get_affor_shares(assetId,price,money=None,volumeStep=None):
     global inbacktest
     global bts
     if inbacktest:
         #print('Esta em backtest. bts=')#, bts)
-        if money==None:
+        if money is None:
             money=bts['capital']
         return pget_affor_shares(assetId,price,money,volumeStep)
     #else:
       #  print('NAO esta em backtest')
-    pget_affor_shares(assetId,money,price,volumeStep)
+    return pget_affor_shares(assetId,price,money,volumeStep)
 
 def pget_affor_shares(assetId,price,money=None,volumeStep=None):
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
     if money is None:
-        money=mt5.account_info().balance
+        money=mt5.account_info().margin_free
     if money <=0:
         return 0.0
         
@@ -304,36 +313,46 @@ def pget_affor_shares(assetId,price,money=None,volumeStep=None):
         step=get_volume_step(assetId)
     else:
         step=volumeStep    
-
-
     free=0
     while free*close<money:
         free=free+step
+        #print('free=',free, ' close=',close,' money=',money  )
     return free-step
 
-""" 
- Returns the account balance in the default currency of the stock
-"""
+
 def get_balance():
+    """ 
+ Returns the Account balance (free resource) in the default currency of the stock
+        It is equivalent to free margin in MT5 jargon
+"""
+    if not connected:
+        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
+        return
     global inbacktest
+    global bts
     if inbacktest:
         #print('Esta em backtest. bts=')#, bts)
         return backtest.get_balance(bts)
     #else:
       #  print('NAO esta em backtest')
-    return mt5.account_info().balance
+    return mt5.account_info().margin_free
 
 
-"""
+
+
+
+def get_position_value(symbolId=None):
+    """
  Returns the current value in portfolio of a given symbol, or total value of alls symbol, if None is given
 """
-def get_position_value(symbolId=None):
     pos=get_positions(symbolId)
     if len(pos)>0:
         return sum(pos['volume']*pos['price_current'])
     else:
         return 0.0
-"""
+
+def get_positions(symbolId=None): 
+    """
  Returns a pandas dataframe with the position of a given symbol or group of symbols.
     If the parameter is None, it returns all current positions in the account
 #Example:
@@ -344,7 +363,6 @@ def get_position_value(symbolId=None):
     pos['price_current']  #current price of the asset
  #print("get position")
 """
-def get_positions(symbolId=None): 
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -360,61 +378,66 @@ def get_positions(symbolId=None):
         return pd.DataFrame()
 
 
-"""
+
+def buyOrder(symbolId,volume,price=None,sl=None,tp=None): # Buying !!
+    """
 Creates and returns a buy order with the given symbolId, volume, price [optional], stop loss[optional], take profit [optional]
 """
-def buyOrder(symbolId,volume,price=None,sl=None,tp=None): # Buying !!
-   if not connected:
+    if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
-   symbol_info = mt5.symbol_info(symbolId)
+    symbol_info = mt5.symbol_info(symbolId)
    #print("symbol=",symbolId," info=",symbol_info)
-   if symbol_info is None:
+    if symbol_info is None:
         setLastError(symbolId + " not found, can not create buy order")
         return None
  
 # se o símbolo não estiver disponível no MarketWatch, adicionamo-lo
-   if not symbol_info.visible:
-        #print(symbolId, "is not visible, trying to switch on")
-        if not mt5.symbol_select(symbolId,True):
-            setLastError("symbol_select({}}) failed! symbol=" +symbolId)
-            return None   
-   #point = mt5.symbol_info(symbolId).point
-   deviation = 20
-   
-   request = {
-    "action": mt5.TRADE_ACTION_DEAL,
-    "symbol": symbolId,
-    "volume": float(volume),
-    "type": mt5.ORDER_TYPE_BUY,
+    if not symbol_info.visible:
+            #print(symbolId, "is not visible, trying to switch on")
+            if not mt5.symbol_select(symbolId,True):
+                setLastError("symbol_select({}}) failed! symbol=" +symbolId)
+                return None   
+    #point = mt5.symbol_info(symbolId).point
+    deviation = 20
+    
+    request = {
+        "action": mt5.TRADE_ACTION_DEAL,
+        "symbol": symbolId,
+        "volume": float(volume),
+        "type": mt5.ORDER_TYPE_BUY,
 
-    "deviation": deviation,
-    "magic": random.randrange(100,100000),
-    "comment": "order by mt5se",
-    "type_time": mt5.ORDER_TIME_GTC,
-    "type_filling": mt5.ORDER_FILLING_RETURN,
-    }
-   if price is None:  # order a mercado
-       request['action']=mt5.TRADE_ACTION_DEAL
-       request['type']=mt5.ORDER_TYPE_BUY
-       request['price']=mt5.symbol_info_tick(symbolId).ask
-   else:  # order limitada
-       request['action']=mt5.TRADE_ACTION_PENDING
-       request['type']=mt5.ORDER_TYPE_BUY_LIMIT
-       request['price']=float(price)
-   if sl is not None:
-       request["sl"]=sl
-   if tp is not None:
-        request["tp"]= tp
-  
+        "deviation": deviation,
+        "magic": random.randrange(100,100000),
+        "comment": "order by mt5se",
+        "type_time": mt5.ORDER_TIME_GTC,
+        "type_filling": mt5.ORDER_FILLING_RETURN,
+        }
+    if price is None:  # order a mercado
+        request['action']=mt5.TRADE_ACTION_DEAL
+        request['type']=mt5.ORDER_TYPE_BUY
+        request['price']=mt5.symbol_info_tick(symbolId).ask
+    else:  # order limitada
+        request['action']=mt5.TRADE_ACTION_PENDING
+        request['type']=mt5.ORDER_TYPE_BUY_LIMIT
+        request['price']=float(price)
+    if sl is not None:
+        request["sl"]=sl
+    if tp is not None:
+            request["tp"]= tp
+    
 
-   return request
+    return request
 
 
-"""
+
+def sellOrder(symbolId,volume,price=None,sl=None,tp=None): # Selling !!
+    """
 Creates and returns a sell order with the given symbolId, volume, price [optional], stop loss[optional], take profit [optional]
 """
-def sellOrder(symbolId,volume,price=None,sl=None,tp=None): # Selling !!
+    if not connected:
+        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
+        return
     symbol_info = mt5.symbol_info(symbolId)
     #print("symbol=",symbolId," info=",symbol_info)
     if symbol_info is None:
@@ -454,11 +477,12 @@ def sellOrder(symbolId,volume,price=None,sl=None,tp=None): # Selling !!
         request["tp"]= tp
     return request
 
-"""
+
+def isSellOrder(req):
+    """
     Returns true if it is a sell order, False if it is a buy order.
         It sets a descriptive error message (getLastError / setLastError) if there is an error
 """
-def isSellOrder(req):
     if req is None:
         print("Error! Order is None!!!!")
         return False
@@ -471,11 +495,30 @@ def isSellOrder(req):
         return False
 
 
+
+def isBuyOrder(req):
+    """
+    Returns true if it is a sell order, False if it is a buy order.
+        It sets a descriptive error message (getLastError / setLastError) if there is an error
 """
+    if req is None:
+        print("Error! Order is None!!!!")
+        return False
+    if req['type']==mt5.ORDER_TYPE_SELL_LIMIT or req['type']==mt5.ORDER_TYPE_SELL:
+        return False
+    elif req['type']==mt5.ORDER_TYPE_BUY_LIMIT or req['type']==mt5.ORDER_TYPE_BUY:
+        return True
+    else:
+        print("Error! Order is not buy our sell!!!!")
+        return False
+
+
+
+def checkOrder(req):
+    """
     Returns true if it a valid order
     It returns false in case of Short order or insufficient money to buy
 """
-def checkOrder(req):
     global inbacktest
     if inbacktest:
         #print('Esta em backtest. bts=')#, bts)
@@ -510,10 +553,11 @@ def checkOrder(req):
 
 
 lastErrorText=""
-"""
+
+def getLastError():
+    """
     Returns a string with the last error's message
 """
-def getLastError():
     global lastErrorText
     if lastErrorText is None or lastErrorText=="":
         return mt5.last_error()
@@ -521,19 +565,54 @@ def getLastError():
         aux=lastErrorText
         lastErrorText=None  
         return aux    
-"""
-    Sets the last error's message
-"""   
+ 
 def setLastError(error):
+    """
+    Sets the last error's message. Used together with getLastError
+"""  
     global lastErrorText
     lastErrorText=error
 
+lastOrderResult=""
+def setLastOrderResult(order,result):
+    """"
+       set the result of the last order. Used together with getLastOrderResult
+            result is a dictionary with four keys:
+            'symbol'  - asset id
+            'isSellOrder' - True if it is a sell order, false if it is a buy order
+            'shares'   - Number of shares of the order
+            'price'    -  Executed price for the order
+    """
+    global lastOrderResult
+    lastOrderResult=dict()
+    lastOrderResult['symbol']=order['symbol'] 
+    lastOrderResult['isSellOrder']=isSellOrder(order)
+    lastOrderResult['shares']=result.volume
+    lastOrderResult['price']=result.price
+    return lastOrderResult
+
+
+def getLastOrderResult():
+    """
+    Returns the result of the last succesful sent order
+        result is a dictionary with four keys:
+            'symbol'  - asset id
+            'isSellOrder' - True if it is a sell order, false if it is a buy order
+            'shares'   - Number of shares of the order
+            'price'    -  Executed price for the order
 """
+    global lastOrderResult
+    return lastOrderResult
+
+
+
+
+def sendOrder(order):
+    """
     Sends the given order to online execution 
         returns True if sucessfull and False otherwise
     Obs: This function should be used only for direct control traders
 """
-def sendOrder(order):
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -555,12 +634,14 @@ def sendOrder(order):
        #             print("       traderequest: {}={}".format(tradereq_filed,traderequest_dict[tradereq_filed]))
         return False
     else:
+        setLastOrderResult(order,result)  # set the result of last sucessful order 
         return True
 
-"""
+
+def numOrders(): 
+    """
  Returns the number of active orders
 """
-def numOrders(): 
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -571,14 +652,15 @@ def numOrders():
     else:
         return result
 
-"""
+
+def get_active_orders():  
+    """
     Returns a dataframe with all active orders
 order fields  description:
     #order_id | symbol | type | price | sl | tp | 
     # type: 0 - Buy (Market Order), 1  - Sell (Market Order)
             2 - Buy (LImited Order), 3 - Sell (Limited Order)
 """
-def get_orders():  
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -594,12 +676,13 @@ def get_orders():
         df['type']=dfaux['type']
         return df
 
-"""
+
+def cancel_order(order_id):  
+    """
     Cancels a order specified by its order_id (see get_orders).
         Returns True if the order is sucessfuly cancelled! False otherwise
             If the order does not exist, it also returns False
 """
-def cancel_order(order_id):  
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -638,11 +721,12 @@ def getDailYBars(symbol, start,end=None): # sao inclusas barras com  tempo de ab
        rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
        return rates_frame
 
-"""
-  Returns a serie of returns from bars using open-close prices
+
+def get_returns(bars):
+    """
+  Returns a serie of returns from given bars using open-close prices
     close[i]/open[i]-1
 """
-def get_returns(bars):
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -651,65 +735,74 @@ def get_returns(bars):
         x.append(bars['close'][i]/bars['open'][i]-1)
     return x
 
-"""
-  Returns the last close price from bars
-"""
+
 def get_last(bars): # argumento deve ser bars nao vazia, retorna erro se estiver vazia
+    """
+  Returns the last close price from given bars
+"""
     if bars is None:
         return 0.0
     return bars['close'].iloc[-1]
 
-"""
-  Returns the first open price from bars
-"""
+
 def get_first(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
+    """
+  Returns the first open price from given bars
+"""
     if bars is None:
         return 0
     return bars['open'][0]
-"""
-  Returns the max high price from bars
-"""
+
 def get_max(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
+    """
+  Returns the max high price from given bars
+"""
     if bars is None:
         return 0
     return max(bars['high'])
 
-"""
-  Returns the min low price from bars
-"""
+
 def get_min(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
+    """
+  Returns the min low price from given bars
+"""
     if bars is None:
         return 0
     return max(bars['low'])
 
-"""
-  Returns the last time from bars
-"""
+
 def get_last_time(bars): # argumento deve ser bars nao vazia, retorna erro se estiver vazia
+    """
+  Returns the last time from given bars
+"""
     if bars is None:
         return 0
     return bars['time'].iloc[-1]
-"""
-  Returns the first time from bars
-"""
+
 def get_first_time(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
+    """
+  Returns the first (earlier) time from given bars
+"""
     if bars is None:
         return 0
     return bars['time'][0]
 
-"""
+
+def read_bars_file(fileName):
+    """
  Returns a pandas data frame with the content of the bars file given as parameter.
     Bars files are csv files that can be obtained from MetaTrader with the following nine columns:
      date,time,open,high,low,close,vol, tickvol,spread
 """
-def read_bars_file(fileName):
     df=pd.read_csv(fileName,delimiter='\t',names=['date','time','open','high','low','close','vol', 'tickvol','spread'],header=0) 
     if df is None or len(df.columns)!=9:
         print("The bars file should be a csv file with nine columns: date,time,open,high,low,close,vol, tickvol,spread")
         return None
     else:
         return df
-"""
+
+def get_bars(symbol, start,end=None,timeFrame=DAILY):
+    """
  Returns a pandas data frame with bars information. Parameters
     symbol - the asset symbol
     start - the start date or the number of the last x desired bars
@@ -719,7 +812,6 @@ def read_bars_file(fileName):
         se.get_bars('AAPL',10) # returns the last 10 bars    
         time   open   high    low  close  tick_volume  spread  real_volume
 """
-def get_bars(symbol, start,end=None,timeFrame=DAILY):
  # definimos o fuso horário como UTC
     #timezone = pytz.timezone("Etc/UTC")
     if not connected:
@@ -767,17 +859,149 @@ def get_bars(symbol, start,end=None,timeFrame=DAILY):
 
 
 
-"""
+
+def get_multi_bars(assets,start,end=None,type=DAILY):
+    """
 Returns bars for multiple assets. It is similar to get_bars (that deals with just one asset)
     mbars=get_multi_bars(assets,start,end)
     mbars[assets[0]] # bars for the first asset (0)
 """
-def get_multi_bars(assets,start,end=None,type=DAILY):
     dbars=dict()
     for asset in assets:
-        dbars[asset]=se.get_bars(asset,start,end,type)
+        dbars[asset]=get_bars(asset,start,end,type)
     return dbars
 
+
+
+
+def get_orders(start,end,allFields=False):
+    """
+Returns list of orders for a given period of time  (start-end)as pandas DataFrame
+    Parameters: start,end (datetime from datetime)
+                allFields=False (by default only the main fields are returned)
+"""
+    if not connected:
+        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
+        return
+    lis2=mt5.history_orders_get(start,end)
+    df2=pd.DataFrame(list(lis2),columns=lis2[0]._asdict().keys())
+    if not allFields:
+        df2.drop(['time_setup_msc','time_done_msc','time_setup_msc','time_expiration','type_time','state','position_by_id','reason','volume_current','price_stoplimit','sl','tp'], axis=1, inplace=True)
+    df2['time_setup'] = pd.to_datetime(df2['time_setup'], unit='s')
+    df2['time_done'] = pd.to_datetime(df2['time_done'], unit='s')
+    #df2['type'] =df2['type'].map(type_order)
+    #df2['reason'] =df2['reason'].map(reason)
+    return df2
+
+
+
+def get_deals(start,end,allFields=False):
+    """
+Returns list of deals for a given period of time  (start-end)as pandas DataFrame
+    deals - include executed orders and orders executed by the broker (margin calls, withdraws, deposits and others)
+    Parameters: start,end (datetime from datetime)
+                allFields=False (by default only the main fields are returned)
+"""
+    if not connected:
+        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
+        return
+    lis2=mt5.history_deals_get(start,end)
+    df2=pd.DataFrame(list(lis2),columns=lis2[0]._asdict().keys())
+    if not allFields:
+        df2.drop(['entry','swap','external_id','time_msc','magic','order','position_id'], axis=1, inplace=True) 
+    df2['time']=pd.to_datetime(df2['time'], unit='s')
+    #df2['type'] =df2['type'].map(type_order)
+    #df2['reason'] =df2['reason'].map(reason)
+    return df2
+
+
+
+def order_type(x):
+    """
+    Returns a string correspondig to the order type (or unkonw). 
+    Possible values and Meaning:
+     'BUY' - Market Buy order
+   'SELL' -  Market Sell order
+    'LIMTED_BUY' -     Buy Limit pending order
+    'LIMITED_SELL' -    Sell Limit pending order
+    'ORDER_TYPE_BUY_STOP' -     Buy Stop pending order
+    'ORDER_TYPE_SELL_STOP' -  Sell Stop pending order
+    'ORDER_TYPE_BUY_STOP_LIMIT' -    #Upon reaching the order price, a pending Buy Limit order is placed at the StopLimit price
+    'ORDER_TYPE_SELL_STOP_LIMIT' -     #Upon reaching the order price, a pending Sell Limit order is placed at the StopLimit price
+    'ORDER_TYPE_CLOSE_BY' -     #Order to close a position by an opposite one
+    'UNKNOWN'  - Probably an error!!
+"""
+    if x==0:
+        #Market Buy order
+        return 'BUY'
+    elif x==1:
+        #Market Sell order
+        return 'SELL'
+    elif x==2: 
+        #Buy Limit pending order
+        return 'LIMTED_BUY'
+    elif x==3:
+        #Sell Limit pending order
+        return 'LIMITED_SELL'
+    elif x==4:
+        #Buy Stop pending order
+        return 'ORDER_TYPE_BUY_STOP'
+    elif x==5:
+        #Sell Stop pending order
+        return 'ORDER_TYPE_SELL_STOP'
+    elif x==6:
+        #Upon reaching the order price, a pending Buy Limit order is placed at the StopLimit price
+        return 'ORDER_TYPE_BUY_STOP_LIMIT'
+    elif x==7:
+        #Upon reaching the order price, a pending Sell Limit order is placed at the StopLimit price
+        return 'ORDER_TYPE_SELL_STOP_LIMIT'
+    elif x==8:
+        #Order to close a position by an opposite one
+        return 'ORDER_TYPE_CLOSE_BY'
+    else:
+        return 'UNKNOWN'
+
+def order_reason(x):
+    """
+    Returns a string correspondig to the order reason (or unkonw)
+"""
+    if x==0:
+        #Transação realizada como resultado da ativação de uma ordem colocada a partir de um terminal desktop
+        return 'DEAL_REASON_CLIENT'
+    elif x==1:
+        #Transação realizada como resultado da ativação de uma ordem colocada a partir de um aplicativo móvel
+        return 'DEAL_REASON_MOBILE'
+    elif x==2:
+        #Transação realizada como resultado da ativação de uma ordem colocada a partir da plataforma web
+        return 'DEAL_REASON_WEB'
+    elif x== 3:
+        #Transação realizada como resultado da ativação de uma ordem colocada a partir de um programa MQL5, Expert Advisor ou script
+        return 'DEAL_REASON_EXPERT'
+    elif x== 4:
+        #Transação realizada como resultado da ativação de uma ordem Stop Loss
+        return 'DEAL_REASON_SL'
+    elif x== 5:
+        #Transação realizada como resultado da ativação de uma ordem Take Profit
+        return 'DEAL_REASON_TP'
+    elif x== 6:
+        #Transação realizada como resultado do evento Stop Out
+        return 'DEAL_REASON_SO'
+    elif x== 7:
+        #Transação realizada devido à transferência da posição
+        return 'DEAL_REASON_ROLLOVER'
+    elif x== 8:
+        #Transação realizada após creditada/debitada a margem de variação
+        return 'DEAL_REASON_VMARGIN'
+    elif x== 9:
+        #Transação realizada após o fracionamento (redução do preço) do instrumento que tinha a posição aberta durante o fracionamento    
+        return 'DEAL_REASON_SPLIT'
+    else:
+        return 'UNKNOWN'
+
+
+
+
+
 """
 Returns intraday (1 minute) bars.
      It is the same as se.get_bars(symbol,day,timeFrame=se.INTRADAY)
@@ -803,8 +1027,10 @@ def roll_bars(bars, new_bars):
 
 
 
-"""
- Returns a pd.DataFrame like the one below for a group of assets. It is similar to get_multi_bars.
+
+def get_close_prices(assets,start,end=None,timeFrame=DAILY):
+    """
+ Returns a pandas.DataFrame like the one below for a group of assets. It is similar to get_multi_bars.
                 XOM        RRC        BBY         MA        PFE        JPM
 date
 2010-01-04  54.068794  51.300568  32.524055  22.062426  13.940202  35.175220
@@ -813,7 +1039,6 @@ date
 ..
 # Note that 'date' column is the index, the others are assets' close prices
 """
-def get_close_prices(assets,start,end=None,timeFrame=DAILY):
     if not connected:
         print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
         return
@@ -827,7 +1052,9 @@ def get_close_prices(assets,start,end=None,timeFrame=DAILY):
         df[asset]=bars['close']
     df=df.set_index('date')
     return df
-"""
+
+def get_close_prices_from_dbars(assets,dbars):
+    """
 Returns a pd.DataFrame like the one below for a group of assets from given multi asset bars
     It is similar to get_multi_bars.
                 XOM        RRC        BBY         MA        PFE        JPM
@@ -839,7 +1066,6 @@ date
 2010-01-08  54.358093  52.597733  32.297466  21.945297  13.756095  36.677460
  Note that 'date' column is the index, the others are assets' close prices
 """
-def get_close_prices_from_dbars(assets,dbars):
     df=pd.DataFrame()
     first=True
     for asset in assets:
@@ -851,11 +1077,12 @@ def get_close_prices_from_dbars(assets,dbars):
     df=df.set_index('date')
     return df
 
-"""
-returns the historical mean, it can be geometric (default) or arithmetic
+
+def mean_historical_return(df,geometric=True):
+    """
+    Returns the historical mean, it can be geometric (default) or arithmetic
     If the data is daily, the mean is daily.
 """
-def mean_historical_return(df,geometric=True):
     expected_returns=dict()
     assets=df.keys()
     for asset in assets:
@@ -868,55 +1095,62 @@ def mean_historical_return(df,geometric=True):
         else:
             ret=(df[asset][size-1]/df[asset][0])
             expected_returns[asset]=ret**(1/(size-1))-1         
-    return pd.Series(expected_returns)
+    return expected_returns
+
 
 
-"""
-    Returns a dictionary with the last prices of set of assets
-"""
 def get_last_prices(assets,dbars=None):
+    """
+    Returns a dictionary with the last prices of set of assets
+        the symbol tickets are the dictionary keys
+"""
     last_prices=dict()
-    for asset in assets:
-        if dbars is None:
+    if dbars is not None:
+        for asset in assets:
             last_prices[asset]=get_last(dbars[asset])
-        else:
+    else:
+        for asset in assets:
             bars=get_bars(asset,1,timeFrame=INTRADAY)
             last_prices[asset]=get_last(bars)
     return last_prices
 
-"""
+
+def get_volume_steps(assets):
+    """
     Returns a dictionary with the volume steps of set of assets
 """
-def get_volume_steps(assets):
     steps=dict()
     for asset in assets:
         steps[asset]=get_volume_step(asset)
     return steps
 
-"""
+
+def get_curr_shares(assets):
+    """
     Returns a dictionary with the current number of shares for a set of assets
 """
-def get_curr_shares(assets):
-    steps=dict()
+    cshares=dict()
     for asset in assets:
-        steps[asset]=get_shares(asset)
-    return steps
+        cshares[asset]=get_shares(asset)
+    return cshares
 
-"""
+
+def orders_from_weights(weights,last_prices,capital):
+    """
     Returns a list of orders to adopt a portfolio defined by a set of weights (dictionary),
         a set of last prices (dictionary) and the amount of available capital 
 """
-def orders_from_weights(assets,weights,last_prices,capital):
     # sort weights from highest to lowest
     weights=dict(sorted(weights.items(), key=lambda item:item[1],reverse=True))
     #round 1 - buy while never exceeds the desired weight
+    assets=weights.keys()
     sum=0
     curr=dict() # current weights
     volumes=dict() #  order's volumes
     steps=get_volume_steps(assets)
     for asset in assets:
         aval_capital=weights[asset]*capital
-        shares=get_affor_shares(asset,aval_capital,last_prices[asset],steps[asset])
+        shares=get_affor_shares(asset,last_prices[asset],aval_capital,steps[asset])
         if shares<=0:
             curr[asset]=0
             volumes[asset]=0
@@ -943,13 +1177,14 @@ def orders_from_weights(assets,weights,last_prices,capital):
             break
     return volumes
 
-"""
+
+def get_new_orders_from_curr_shares(orders,curr_shares):
+    """
     Get new adjusted orders considering current shares of a set of assets, and orders to buy a certain
     numbers of shares for each asset. For instance,
         if orders define buy 1000 of X and 500 of Y and currently the account has 100 shares of X and 900 shares of Y
         the new orders would be buy 900 shares of X and sell 400 shares of Y.
 """
-def get_new_orders_from_curr_shares(orders,curr_shares):
     new_orders=dict()
     for k in orders.keys():
         new_orders[k]=orders[k]-curr_shares[k]
@@ -957,45 +1192,90 @@ def get_new_orders_from_curr_shares(orders,curr_shares):
 
 
 
-"""
+
+def date(year,month,day,hour=0,min=0,sec=0):
+    """
     Returns a datetime with the specified year,month,day,hour=0,min=0,sec=0
 """
-def date(year,month,day,hour=0,min=0,sec=0):
     return datetime(year,month,day,hour,min,sec)
 
 
 ###############################
 # Classes used in inverse control trader and backtest!!
 class Trader:
+    """
+        Basic Trader class for mt5se
+    """
     def __init__(self):
+        """
+            Trader's constructor
+        """
         pass
     # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
     #   and setups the operation
     def setup(self,dbars):
+        """
+            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
+       and prepares the trader for execution. It is called once, just after trader creation
+        """
         pass
-    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
-    #   and returns a dictionary with the order for each asset
-    def analyze(self,dbars):
+
+    def trade(self,dbars):
+        """
+            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
+       and returns a list of orders with up to one order for each asset. It is called every trading cycle
+        """
         pass
-    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
-    #   and frees resources used by the Trader
+
+    def orders_result(self, exec_orders):
+        """
+            Receive a list of executed orders. It is called after Trader.trade() with the list of orders given by Trader.trader that were really executed.
+            Receives orders_res_list - a list of the orders really executed with real volume and price
+       Each result is a dictionary 'order' with:
+        order['symbol'] - string
+        order['isSellOrder'] - boolean
+        order['shares'] - float
+        order['price'] - float
+        It is called once at every trading cycle
+     """
+        pass
+
     def ending(self,dbars):
+        """
+            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
+        It is called once, just before trader's end of execution
+        """
         pass
  
 
-# used for the Trader to get analyzes
 class Analyst:
+    """
+        Basic Analyst class for mt5se. It may be used for Trader to get target prices.
+    """
     def __init__(self):
+        """
+            Analyst's constructor
+        """
         pass
     # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
     #   and setups the operation
     def setup(self,dbars):
+        """
+            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
+       and prepares the analyst for execution. It is called once, just after trader creation
+        """
         pass
-    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
-    #   and returns a dictionary with the annualized expected returns for each asset
+
     def analyze(self,dbars):
+        """
+            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
+       and returns a list of target price for each asset (-1 if not available for a given asset). It is called every trading cycle
+        """
         pass
-    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
-    #   and frees resources used by the Analyst
+   
     def ending(self,dbars):
+        """
+            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
+        It is called once, just before analyst's end of execution
+        """
         pass
\ No newline at end of file
