@@ -10,7 +10,7 @@ from scipy import stats
 
 def rsi(returns):
     if type(returns)==pd.core.frame.DataFrame:
-        returns=se.getReturns(returns)
+        returns=se.get_returns(returns)
     u=0.0
     uc=0
     d=0.0
@@ -35,7 +35,7 @@ def rsi(returns):
 #      for a serie of prices in regular intervals
 def slope(serie):
     x=np.array(range(len(serie)))
-    y=np.array(serie)
+    #y=np.array(serie)
     s=stats.linregress(x,serie)
     return s.slope
 
